/*----------------------------------------------
   MegaStorm Tile-based Engine - Roberto Prieto
   Copyright (C) 2007-2012 MegaStorm Systems
  ----------------------------------------------
    
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
     
    Roberto Prieto
    contact@megastormsystems.com
    http://www.megastormsystems.com

    Used software licenses, including the one for this engine, are located
    at /licenses directory. Please, include them on your distributions.

 --------------------------------------------------------------------------
             MSTE Tools(for internal usage) header file
    
    Changelog: read MSTE_Tools.h to see all changes
   
 -------------------------------------------------------------------------- */
#include <time.h>
#include <stdio.h>
#include <math.h>
#include "zlib/zlib.h"
#include "MSTE_Tools.h"

// Comparing two 'tiles' on the given surface: supporting colorkey and per-pixel surfaces.
// With alpha per-pixel surfaces, colorkey is useless
// Return 0 -> Diff tile
// Return 1 -> Same tile
// Return 2 -> Transparent tile
int CompareTiles(_sTileProcess *t1, _sTileProcess *t2, SDL_Surface *surface,int maxdif,int sizeX, int sizeY, int colorkey) 
{	
    int x,y;
    char bFlagColorkey, bFlagPerPixel; 
    int pixels = 0; 
    Uint8 r,g,b,a;
    unsigned int p1,p2;

    // 1.1.Without colorkey, we asume it is the same tile
    if(colorkey == -1) bFlagColorkey = 1;
    // 1.2.Otherwise, we assume it is a transparent tile
    else bFlagColorkey = 2;
    // 1.3.With per-pixel, we asume it is a transparent tile
    if((surface->flags & SDL_SRCALPHA) && (surface->format->alpha == 255)) bFlagPerPixel = 2;
    // 1.4.Otherwise, we assume it is the same tile
    else bFlagPerPixel = 1;

    // 2.Comparing tiles
    for(y = 0; y < sizeY; y++)
        for(x = 0; x < sizeX; x++)
        {
            // 2.1.Select right depth color
            switch(surface->format->BytesPerPixel)
            {
                // 8bit mode
            case 1:
                {
                    p1 = *((Uint8*)surface->pixels + (t1->posy+y) * surface->pitch + (t1->posx+x));
                    p2 = *((Uint8*)surface->pixels + (t2->posy+y) * surface->pitch + (t2->posx+x));					
                    break;
                }

                // 16bit mode
            case 2:
                {
                    p1 = *((Uint16*)surface->pixels + (t1->posy+y) * (surface->pitch >> 1) + (t1->posx+x));
                    p2 = *((Uint16*)surface->pixels + (t2->posy+y) * (surface->pitch >> 1) + (t2->posx+x));
                    break;
                }

                // 24bit mode
            case 3:
                {
                    // Not supported
                    return 0;
                    break;
                }	

                // 32bit mode
            case 4:
                {
                    p1 = *((Uint32*)surface->pixels + (t1->posy+y) * (surface->pitch >> 2) + (t1->posx+x));
                    p2 = *((Uint32*)surface->pixels + (t2->posy+y) * (surface->pitch >> 2) + (t2->posx+x));
                    SDL_GetRGBA(p2,surface->format, &r, &g, &b, &a);
                    if(a != 0) bFlagPerPixel = 1;
                    break;
                }
            }

            // 2.2.Is this tile transparent? Checking colorkey...
            if(colorkey != -1)
            {
                if(p2 != colorkey) bFlagColorkey = 1; // -> NOT!
            }

            // 2.3.Compare both pixels!
            if( p1 !=  p2 ) 
            {
                if(pixels >= maxdif) return 0;
                else pixels++;
            }
        }

        // 3.Return
        return  (bFlagColorkey > bFlagPerPixel) ? bFlagColorkey : bFlagPerPixel;
}

// Remove a tile from the given DPF
// Return 0 Failed
//        1 Succeed
int RemoveTile(char *dpfname, char *blockname)
{
    DPF_BlockData block;
    int idDPF,ret;

    // Checks
    if((dpfname == NULL) || (blockname == NULL)) return 0;
    idDPF = IStuffDPF->Open(dpfname);
    if(idDPF == 0) return 0;	
    strcpy(block.name,blockname);
    strcpy(block.type,"TILE2");
    ret = IStuffDPF->Delete(idDPF,&block);
    IStuffDPF->Close(idDPF);
    return ret;
}

// Convert a tileset with margin!=0 & spacing!=0 to margin=0 & spacing=0
// Return NULL if failed
// or a pointer to a created tileset if succeed
CRM32Pro_CTile *ConvertToTileset(char *sIMGfile, char *sTileName, int iTileWidth, int iTileHeight, int iSpacing, int iMargin)
{
    CRM32Pro_CTile *myTile = NULL;
    SDL_Surface *sSrc = NULL, *sDst = NULL;
    int iNumWT = 0, iNumHT = 0;
    int i,j, iAlphaPerPixel = 0;
    SDL_Rect rSrc, rDst;

    // 1.Try to load the image
    sSrc = IImage->LoadIMG(sIMGfile);
    if(sSrc == NULL) return NULL;

    // 2.Some calcs and creation of our surface
    i = iMargin + iTileWidth;
    while(i <= sSrc->w - iMargin)
    {
        iNumWT++;
        i += iTileWidth + iSpacing;
    }
    i = iMargin + iTileHeight;
    while(i <= sSrc->h - iMargin)
    {
        iNumHT++;
        i += iTileHeight + iSpacing;
    }
    if((sSrc->flags & SDL_SRCALPHA) && (sSrc->format->alpha == 255)) 
    {
        iAlphaPerPixel = 1;
        sDst = SDL_CreateRGBSurface(SDL_SWSURFACE,iTileWidth * iNumWT, iTileHeight * iNumHT,sSrc->format->BitsPerPixel,sSrc->format->Rmask,sSrc->format->Gmask,sSrc->format->Bmask,sSrc->format->Amask);
    }
    else
        sDst = SDL_CreateRGBSurface(SDL_SWSURFACE,iTileWidth * iNumWT, iTileHeight * iNumHT, 32, 0, 0, 0, 0);

    // 3.Convert the surfaces
    rSrc.w = rDst.w = iTileWidth;
    rSrc.h = rDst.h = iTileHeight;
    for(j = 0; j <= iNumHT; j++)
    {
        rSrc.y = iMargin + (j * iSpacing) + (j * iTileHeight);
        rDst.y = j * iTileHeight;
        for(i = 0; i <= iNumWT; i++)
        {
            rSrc.x = iMargin + (i * iSpacing) + (i * iTileWidth);
            rDst.x = i * iTileWidth;
            if(iAlphaPerPixel)
            {
                int saved_flags = sSrc->flags&(SDL_SRCALPHA|SDL_RLEACCELOK);
                int saved_alpha = sSrc->format->alpha;
                SDL_SetAlpha(sSrc, 0, 0);
                SDL_BlitSurface(sSrc,&rSrc,sDst,&rDst);
                SDL_SetAlpha(sSrc, saved_flags, saved_alpha);
            }
            else SDL_BlitSurface(sSrc,&rSrc,sDst,&rDst);
        }
    }

    // 4.Create the new tileset
    myTile = new CRM32Pro_CTile();
    myTile->Create(sDst,sTileName);
    myTile->SetTileSet(1,iTileWidth,iTileHeight);
    //IImage->SavePNG("out.png",sDst);

    // 5.Remove resources and return
    SDL_FreeSurface(sSrc);
    SDL_FreeSurface(sDst);
    return myTile;
}

// Check given sFile name for the sExt (file extension)
// Return 0 No
// Return 1 Yes
int CheckFileExtension(char *sFile, char *sExt)
{
    if(sFile == NULL || sExt == NULL) return 0;

    // Have we an extension on the filename?
    if(strchr(sFile, '.') == NULL || strchr(sExt, '.') == NULL) return 0;

    // Check for the extension
    for(int i = 0; i < (int)strlen(sFile); i++)
    {
        if(tolower(sFile[strlen(sFile) - i - 1]) == tolower(sExt[strlen(sExt) - i - 1]))
        {
            if(i == strlen(sExt) - 1) return 1;
        }
        else break;
    }

    return 0;
}

// Check if a file exists
int fexists(const char *filename)
{
    FILE *fp;

    fp = fopen(filename,"rb");
    if(fp)
    {
        fclose(fp);
        return 1;
    }
    return 0;
}

// Copy string-zero of n size avoiding buffer overflow and always appending a NULL string termination
int szCopy(char *dst, const char *src, int n)
{
    if((int)strlen(src) > n - 1) { strncpy(dst,src,n - 1); dst[n - 1] = (char)'\0'; }
    else strcpy(dst,src); 
    return 1;
}

// Get a string with date/time stamp
char *GetDateStamp()
{
    char *szOutput;
    long              ltime;
    const struct tm  *localt;
    time(&ltime);
    localt = localtime(&ltime);
    szOutput = asctime(localt);
    // Remove the CR
    szOutput[strlen(szOutput) - 1] = (char)'\0';
    // Return
    return szOutput;
}

// Round a float value
float round(float num) 
{
    return (num > 0.0) ? floor(num + 0.5f) : ceil(num - 0.5f);
}

// Write a TMX property
int WriteTMXProperty(int idXML, char *name, char *value)
{
    int iRet = 0;

    iRet = CRM32Pro.XMLNodeCreate(idXML,"property");
    iRet = CRM32Pro.XMLAttributeSet(idXML,"name",name);	
    iRet = CRM32Pro.XMLAttributeSet(idXML,"value",value);	
    CRM32Pro.XMLNodeParent(idXML);

    return iRet;
}
int WriteTMXProperty(int idXML, char *name, int value)
{
    int iRet = 0;

    iRet = CRM32Pro.XMLNodeCreate(idXML,"property");
    iRet = CRM32Pro.XMLAttributeSet(idXML,"name",name);	
    iRet = CRM32Pro.XMLAttributeSet(idXML,"value",value);	
    CRM32Pro.XMLNodeParent(idXML);

    return iRet;
}
int WriteTMXProperty(int idXML, char *name, double value)
{
    int iRet = 0;

    iRet = CRM32Pro.XMLNodeCreate(idXML,"property");
    iRet = CRM32Pro.XMLAttributeSet(idXML,"name",name);	
    iRet = CRM32Pro.XMLAttributeSet(idXML,"value",value);	
    CRM32Pro.XMLNodeParent(idXML);

    return iRet;
}

// Read a TMX property
int ReadTMXProperty(int idXML, char *name, char **value)
{
    int   iRet = 0;
    char *szTmp = NULL;

    // We are inside a "properties" node, get into and looking for "property" subnodes till we match the name
    int retvalue = CRM32Pro.XMLNodeChild(idXML);
    do
    {
        // Is it a property node?
        if(!strcmp(CRM32Pro.XMLNodeGetName(idXML),"property"))
        {
            if(CRM32Pro.XMLAttributeGet(idXML,"name",&szTmp))
            {
                if(!strcmp(name,szTmp))
                {
                    if(CRM32Pro.XMLAttributeGet(idXML,"value",value))
                    {
                        iRet = 1;
                        break;
                    }
                }
            }
        }
    } while(CRM32Pro.XMLNodeNext(idXML)); // next property

    // Go back if we entered on the property subnode
    if(retvalue) CRM32Pro.XMLNodeParent(idXML);

    return iRet;
}
int ReadTMXProperty(int idXML, char *name, int *value)
{
    int   iRet = 0;
    char *szTmp = NULL;

    // We are inside a "properties" node, get into and looking for "property" subnodes till we match the name
    int retvalue = CRM32Pro.XMLNodeChild(idXML);
    do
    {
        // Is it a property node?
        if(!strcmp(CRM32Pro.XMLNodeGetName(idXML),"property"))
        {
            if(CRM32Pro.XMLAttributeGet(idXML,"name",&szTmp))
            {
                if(!strcmp(name,szTmp))
                {
                    if(CRM32Pro.XMLAttributeGet(idXML,"value",value))
                    {
                        iRet = 1;
                        break;
                    }
                }
            }
        }
    } while(CRM32Pro.XMLNodeNext(idXML)); // next property

    // Go back if we entered on the property subnode
    if(retvalue) CRM32Pro.XMLNodeParent(idXML);

    return iRet;
}
int ReadTMXProperty(int idXML, char *name, double *value)
{
    int   iRet = 0;
    char *szTmp = NULL;

    // We are inside a "properties" node, get into and looking for "property" subnodes till we match the name
    int retvalue = CRM32Pro.XMLNodeChild(idXML);
    do
    {
        // Is it a property node?
        if(!strcmp(CRM32Pro.XMLNodeGetName(idXML),"property"))
        {
            if(CRM32Pro.XMLAttributeGet(idXML,"name",&szTmp))
            {
                if(!strcmp(name,szTmp))
                {
                    if(CRM32Pro.XMLAttributeGet(idXML,"value",value))
                    {
                        iRet = 1;
                        break;
                    }
                }
            }
        }
    } while(CRM32Pro.XMLNodeNext(idXML)); // next property

    // Go back if we entered on the property subnode
    if(retvalue) CRM32Pro.XMLNodeParent(idXML);

    return iRet;
}


// Decode base64 strings
static inline bool is_base64(unsigned char c) 
{
    return (isalnum(c) || (c == '+') || (c == '/'));
}
static const std::string base64_chars =
"ABCDEFGHIJKLMNOPQRSTUVWXYZ"
"abcdefghijklmnopqrstuvwxyz"
"0123456789+/";
std::string decodeBase64(std::string const& encoded_string)
{
    int in_len = encoded_string.size();
    int i = 0;
    int j = 0;
    int in_ = 0;
    unsigned char char_array_4[4], char_array_3[3];
    std::string ret;

    while(in_len-- && ( encoded_string[in_] != '=') && is_base64(encoded_string[in_])) 
    {
        char_array_4[i++] = encoded_string[in_]; in_++;
        if(i == 4) 
        {
            for(i = 0; i < 4; i++) char_array_4[i] = base64_chars.find(char_array_4[i]);
            char_array_3[0] = (char_array_4[0] << 2) + ((char_array_4[1] & 0x30) >> 4);
            char_array_3[1] = ((char_array_4[1] & 0xf) << 4) + ((char_array_4[2] & 0x3c) >> 2);
            char_array_3[2] = ((char_array_4[2] & 0x3) << 6) + char_array_4[3];
            for(i = 0; (i < 3); i++) ret += char_array_3[i];
            i = 0;
        }
    }

    if(i) 
    {
        for(j = i; j < 4; j++) char_array_4[j] = 0;
        for(j = 0; j < 4; j++) char_array_4[j] = base64_chars.find(char_array_4[j]);
        char_array_3[0] = (char_array_4[0] << 2) + ((char_array_4[1] & 0x30) >> 4);
        char_array_3[1] = ((char_array_4[1] & 0xf) << 4) + ((char_array_4[2] & 0x3c) >> 2);
        char_array_3[2] = ((char_array_4[2] & 0x3) << 6) + char_array_4[3];
        for(j = 0; (j < i - 1); j++) ret += char_array_3[j];
    }
    return ret;
}


// Decompress a string (supports zlib and gzip formats)
void logZlibError(int error)
{
    switch (error)
    {
    case Z_MEM_ERROR:
        ILogSystem.Msg(LOG_NORMAL,"Out of memory while decompressing data!");
        break;
    case Z_VERSION_ERROR:
        ILogSystem.Msg(LOG_NORMAL,"Incompatible zlib version!");
        break;
    case Z_NEED_DICT:
    case Z_DATA_ERROR:
        ILogSystem.Msg(LOG_NORMAL,"Incorrect zlib compressed data!");
        break;
    default:
        ILogSystem.Msg(LOG_NORMAL,"Unknown error while decompressing data!");
    }
}
char *decompress(std::string const &data, int expectedSize)
{
    int bufferSize = expectedSize;
    int ret;
    z_stream strm;
    char *out = (char *) malloc(bufferSize);

    strm.zalloc = Z_NULL;
    strm.zfree = Z_NULL;
    strm.opaque = Z_NULL;
    strm.next_in = (Bytef *) data.data();
    strm.avail_in = data.length();
    strm.next_out = (Bytef *) out;
    strm.avail_out = bufferSize;

    ret = inflateInit2(&strm, 15 + 32);
    if (ret != Z_OK)
    {
        logZlibError(ret);
        free(out);
        return NULL;
    }

    do
    {
        ret = inflate(&strm, Z_SYNC_FLUSH);
        switch (ret)
        {
        case Z_NEED_DICT:
        case Z_STREAM_ERROR:
            ret = Z_DATA_ERROR;
        case Z_DATA_ERROR:
        case Z_MEM_ERROR:
            inflateEnd(&strm);
            logZlibError(ret);
            free(out);
            return NULL;
        }
        if (ret != Z_STREAM_END)
        {
            out = (char *) realloc(out, bufferSize * 2);
            if (!out)
            {
                inflateEnd(&strm);
                logZlibError(Z_MEM_ERROR);
                free(out);
                return NULL;
            }
            strm.next_out = (Bytef *)(out + bufferSize);
            strm.avail_out = bufferSize;
            bufferSize *= 2;
        }
    }
    while (ret != Z_STREAM_END);

    if (strm.avail_in != 0)
    {
        logZlibError(Z_DATA_ERROR);
        free(out);
        return NULL;
    }
    inflateEnd(&strm);
    return out;
}
