/*----------------------------------------------
   MegaStorm Tile-based Engine - Roberto Prieto
   Copyright (C) 2007-2012 MegaStorm Systems
  ----------------------------------------------
    
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
    
    Roberto Prieto
    contact@megastormsystems.com
    http://www.megastormsystems.com

    Used software licenses, including the one for this engine, are located
    at /licenses directory. Please, include them on your distributions.

 --------------------------------------------------------------------------
             MSTE Tools(for internal usage) header file
    
    Changelog: 
 v1.3 July/2012		- Added zlib_decompress() and decodeBase64() functions
 v1.2 June/2012		- Added round(), WriteTMXProperty(), ReadTMXProperty() and ConvertToTileset() functions
 v1.1 May/2012		- Added szCopy() and GetDateStamp() functions
 v1.0 July/2007		- First version: added generic methods to be used across the MSTE
   
 -------------------------------------------------------------------------- */
#ifndef _MSTE_TOOL_H_
#define _MSTE_TOOL_H_

#include <CRM32Pro.h>

//-------------------------------------------------------------------------------
// MSTE TOOLS global attributes and prototypes
//-------------------------------------------------------------------------------
struct _sTileProcess
{
    int status;
    int posx;
    int posy;
};	

// Comparing tiles. It also detect transparent tiles.
int CompareTiles(_sTileProcess *t1, _sTileProcess *t2, SDL_Surface *surface,int maxdif,int sizeX, int sizeY, int colorkey);

// Remove a given tile
int RemoveTile(char *dpfname, char *blockname);

// Convert a tileset with margin!=0 & spacing!=0 to margin=0 & spacing=0
CRM32Pro_CTile *ConvertToTileset(char *sIMGfile, char *sTileName, int iTileWidth, int iTileHeight, int iSpacing, int iMargin);

// Check given sFile name for the sExt (file extension)
int CheckFileExtension(char *sFile, char *sExt);

// Check if a file exists
int fexists(const char *filename);

// Copy up to n characters appending a NULL string termination
int szCopy(char *dst, const char *src, int n);

// Get a string with date/time stamp
char *GetDateStamp();

// Round a float
float round(float num);

// Write a TMX property
int WriteTMXProperty(int idXML, char *name, char *value);
int WriteTMXProperty(int idXML, char *name, int value);
int WriteTMXProperty(int idXML, char *name, double value);

// Read a TMX property
int ReadTMXProperty(int idXML, char *name, char **value);
int ReadTMXProperty(int idXML, char *name, int *value);
int ReadTMXProperty(int idXML, char *name, double *value);

// Decode base64 string
std::string decodeBase64(std::string const & encoded_string);

// Decompress a string (supports zlib and gzip formats)
char *decompress(std::string const &data, int expectedSize);

//-------------------------------------------------------------------------------
#endif
//-------------------------------------------------------------------------------
// End of MSTE TOOLS header
//-------------------------------------------------------------------------------
